﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Reflection;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.PluginInfrastructure;
using System;

namespace HeuristicLab.BioBoost.Operators.Mutation {
  [StorableClass]
  public class MultiCrossover : StochasticMultiBranch<ICrossover>, ICrossover, IStochasticOperator {

    public override bool CanChangeName {
      get { return false; }
    }

    protected override bool CreateChildOperation {
      get { return true; }
    }

    #region Construction & Cloning
    [StorableConstructor]
    protected MultiCrossover(bool isDeserializing) : base(isDeserializing) {}
    protected MultiCrossover(MultiCrossover orig, Cloner cloner) : base(orig, cloner) {}
    public MultiCrossover() {
      foreach (var t in ApplicationManager.Manager.GetTypes(typeof(ICrossover), Assembly.GetExecutingAssembly())) {
        if (t == this.GetType()) continue; // avoid infinite recursion
        ICrossover instance = null;
        try {
          instance = Activator.CreateInstance(t) as ICrossover;
        } catch {}
        if (instance != null) Operators.Add(instance);
      }
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new MultiCrossover(this, cloner);
    }
    #endregion

    public override IOperation InstrumentedApply() {
        if (Operators.Count == 0) throw new InvalidOperationException(Name + ": Please add at least one crossover to choose from.");
        return base.InstrumentedApply();
    }
  }
}
