﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.BioBoost.Operators.Transformation {

  [StorableClass]
  public class FeedstockTransformer : BioBoostOperator {


    public ValueLookupParameter<StringValue> ProductNameParameter { get { return (ValueLookupParameter<StringValue>) Parameters["ProductName"]; } }

    public string ProductName {
      get { return ProductNameParameter.Value.Value; }
      set { ProductNameParameter.Value = new StringValue(value);}
    }

    #region Construction & Cloning
    [StorableConstructor]
    protected FeedstockTransformer(bool isDeserializing) : base(isDeserializing) {}
    protected FeedstockTransformer(FeedstockTransformer orig, Cloner cloner) : base(orig, cloner) {}
    public FeedstockTransformer() {
      Parameters.Add(new ValueLookupParameter<StringValue>("ProductName", "The product name to be transformed."));
    }
    public override IDeepCloneable  Clone(Cloner cloner) {
      return new FeedstockTransformer(this, cloner);
    }
    #endregion

    public override IOperation Execute(IExecutionContext context, System.Threading.CancellationToken cancellationToken) {
      return base.Execute(context, cancellationToken);
    }

    protected void TransformValue<T>(string valueName, T value) where T : class, IItem {
      IParameter param;
      if (!Parameters.TryGetValue(valueName, out param)) {
        param = new LookupParameter<T>(valueName, "Transformed value.");
        ((ILookupParameter) param).ExecutionContext = ExecutionContext;
        Parameters.Add(param);
      }
      param.ActualValue = value;
    }


    protected double GetMaxCapacity(int i, ValueParameterCollection maxCapacities, double defaultValue) {
      var regionName = ProblemData.LocationNames[i];
      IValueParameter param;
      if (maxCapacities.TryGetValue(regionName, out param)) return ((DoubleValue) param.Value).Value;
      if (maxCapacities.TryGetValue("Default", out param)) return ((DoubleValue) param.Value).Value;
      return defaultValue;
    }
  }
}
